﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Lifetime;
using System.Runtime.Serialization;
using System.Threading;
using System.Security.Permissions;

    public class HelloService : MarshalByRefObject
    {
        public string HelloMethod(string name)
        {
            Console.WriteLine("Hello " + name);
            return "Hello " + name;
        }

        [SecurityPermissionAttribute(SecurityAction.LinkDemand, Flags = SecurityPermissionFlag.Infrastructure)]
        public override object InitializeLifetimeService()
        {
            ILease baseLease = (ILease)base.InitializeLifetimeService();
            if (baseLease.CurrentState == LeaseState.Initial)
            {
                // For demonstration the initial time is kept small.
                // in actual scenarios it will be large.
                baseLease.InitialLeaseTime = TimeSpan.FromSeconds(15);
                baseLease.RenewOnCallTime = TimeSpan.FromSeconds(15);
                baseLease.SponsorshipTimeout = TimeSpan.FromMinutes(2);
            }
            return baseLease;
        }
    }

    class HelloServer
    {
        public static void ServerMain()
        {
            BinaryServerFormatterSinkProvider provider = new BinaryServerFormatterSinkProvider();
            provider.TypeFilterLevel = System.Runtime.Serialization.Formatters.TypeFilterLevel.Full;
            IDictionary props = new Hashtable();
            props["port"] = 12345;

            TcpChannel myChannel = new TcpChannel(props, null, provider);
            ChannelServices.RegisterChannel(myChannel);
            RemotingConfiguration.RegisterActivatedServiceType(typeof(HelloService));
            //RemotingConfiguration.RegisterWellKnownServiceType(typeof(HelloService), "HelloService", WellKnownObjectMode.SingleCall);

            Console.WriteLine("Server started.");
            Console.WriteLine("Hit enter to terminate...");
            Console.Read();
        }
    }


    class HelloClient
    {
        public static void ClientMain()
        {
            // Register the channel.
            BinaryServerFormatterSinkProvider provider = new BinaryServerFormatterSinkProvider();
            provider.TypeFilterLevel = System.Runtime.Serialization.Formatters.TypeFilterLevel.Full;
            IDictionary props = new Hashtable();
            props["port"] = 0; 
            TcpChannel myChannel = new TcpChannel(props, null, provider);
            ChannelServices.RegisterChannel(myChannel);
            RemotingConfiguration.RegisterActivatedClientType(
                                   typeof(HelloService), "tcp://localhost:12345");
            // Create a remote object.
            HelloService myService = new HelloService();
//            HelloService myService = (HelloService)Activator.GetObject(typeof(HelloService), @"tcp://localhost:12345/HelloService");

            TimeSpan myTimeSpan = TimeSpan.FromSeconds(10);


            ILease myLease;
            myLease = (ILease)RemotingServices.GetLifetimeService(myService);
            if (myLease == null)
            {
                Console.WriteLine("Cannot lease.");
                return;
            }

            Console.WriteLine("Initial lease time is " + myLease.InitialLeaseTime);
            Console.WriteLine("Current lease time is " + myLease.CurrentLeaseTime);
            Console.WriteLine("Renew on call time is " + myLease.RenewOnCallTime);
            Console.WriteLine("Sponsorship timeout is " + myLease.SponsorshipTimeout);
            Console.WriteLine("Current lease state is " + myLease.CurrentState.ToString());
            // Register with a sponser.
            ClientSponsor mySponsor = new ClientSponsor();
            myLease.Register(mySponsor);
            Console.WriteLine("Wait for lease to expire (approx. 15 seconds)...");
            System.Threading.Thread.Sleep(myLease.CurrentLeaseTime);
            Console.WriteLine("Current lease time before renewal is " + myLease.CurrentLeaseTime);

            // Renew the lease time.
            myLease.Renew(myTimeSpan);
            Console.WriteLine("Current lease time after renewal is " + myLease.CurrentLeaseTime);

            // Call the Remote method.
            Console.WriteLine("Remote method output is " + myService.HelloMethod("Microsoft"));

            myLease.Unregister(mySponsor);
            GC.Collect();
            GC.WaitForPendingFinalizers();

            // Register with lease time of 15 minutes.
            myLease.Register(mySponsor, TimeSpan.FromMinutes(15));
            Console.WriteLine("Registered client with lease time of 15 minutes.");
            Console.WriteLine("Current lease time is " + myLease.CurrentLeaseTime);

            // Call the Remote method.
            Console.WriteLine("Remote method output is " + myService.HelloMethod("Microsoft"));
            myLease.Unregister(mySponsor);
        }
    }



class Program
{
    static void Main(string[] args)
    {
        try
        {
            HelloServer.ServerMain();
            return;
        }
        catch
        {
        }
        HelloClient.ClientMain();
    }
}
